@@ -163,12 +163,15 @@ static CSONPATH_DO_RET_TYPE csonpath_do_internal(struct csonpath cjp[static 1],
 	case CSONPATH_INST_FILTER_KEY_EQ:
 	case CSONPATH_INST_FILTER_KEY_REG_EQ:
 	case CSONPATH_INST_FILTER_KEY_NOT_EQ:
+	case CSONPATH_INST_FILTER_KEY_SUPERIOR:
+	case CSONPATH_INST_FILTER_KEY_INFERIOR:
 	{
 	    CSONPATH_JSON el;
 	    int operation = cjp->inst_lst[idx].inst;
 	    walker += cjp->inst_lst[idx].next;
 	    char *owalker;
 	    int filter_next = cjp->inst_lst[idx].filter_next;
+	    int operand_instruction = -1;
 	    int in_idx = idx + 1;
 
 	    CSONPATH_DO_FILTER_PRE_LOOP;
@@ -189,18 +192,31 @@ static CSONPATH_DO_RET_TYPE csonpath_do_internal(struct csonpath cjp[static 1],
 			    }
 			    owalker += cjp->inst_lst[idx].next;
 			}
-			
-			if (cjp->inst_lst[idx].inst != CSONPATH_INST_FILTER_OPERAND_STR)
-			    CSONPATH_GETTER_ERR("unsuported instruction '%s' only FILTER_OPERAND_STR is supported here", csonpath_instuction_str[(int)cjp->inst_lst[idx].inst]);
+
+			if (operand_instruction < 0) {
+			    operand_instruction = cjp->inst_lst[idx].inst;
+			}
 
 			if (el2 != CSONPATH_NULL) {
 			    _Bool match = 0;
 			    switch (operation) {
 			    case CSONPATH_INST_FILTER_KEY_NOT_EQ:
-				match = !CSONPATH_EQUAL_STR(el2, owalker);
+				match = !csonpath_do_match(operand_instruction, el2, owalker);
 				break;
 			    case CSONPATH_INST_FILTER_KEY_EQ:
-				match = CSONPATH_EQUAL_STR(el2, owalker);
+				match = csonpath_do_match(operand_instruction, el2, owalker);
+				break;
+			    case CSONPATH_INST_FILTER_KEY_SUPERIOR:
+				if (!CSONPATH_IS_NUM(el2))
+				    break;
+				match = csonpath_int_from_walker(operand_instruction, owalker) <
+				    CSONPATH_GET_NUM(el2);
+				break;
+			    case CSONPATH_INST_FILTER_KEY_INFERIOR:
+				if (!CSONPATH_IS_NUM(el2))
+				    break;
+				match = csonpath_int_from_walker(operand_instruction, owalker) >
+				    CSONPATH_GET_NUM(el2);
 				break;
 			    case CSONPATH_INST_FILTER_KEY_REG_EQ:
 #ifdef CSONPATH_NO_REGEX
@@ -299,7 +315,10 @@ static CSONPATH_DO_RET_TYPE csonpath_do_internal(struct csonpath cjp[static 1],
 		    break;
 	    }
 	default:
-	    CSONPATH_GETTER_ERR("unimplemented %d at idx %d\n", cjp->inst_lst[idx].inst, idx);
+	    CSONPATH_GETTER_ERR("unimplemented %d (%s) at idx %d\n", cjp->inst_lst[idx].inst,
+				cjp->inst_lst[idx].inst <= CSONPATH_INST_BROKEN ?
+				csonpath_instuction_str[cjp->inst_lst[idx].inst] : "(unknow)",
+				idx);
 	}
       next_inst:
 	++idx;
