@@ -20,9 +20,27 @@
 
 #define CSONPATH_IS_STR(o) PyUnicode_Check(o)
 
+#define CSONPATH_IS_NUM(o) PyLong_Check(o)
+
 #define CSONPATH_GET_STR(obj)			\
     PyUnicode_AsUTF8(obj)
 
+#define CSONPATH_DECREF(obj)			\
+  Py_DECREF(obj);
+  
+
+#define CSONPATH_GET_NUM(obj)			\
+    PyLong_AsLong(obj)
+
+#define CSONPATH_EQUAL_NUM(obj, to_cmp)		\
+    ({						\
+	_Bool r = 0;				\
+	if (PyLong_Check(obj)) {		\
+	    r = PyLong_AsLong(obj) == to_cmp;	\
+	}					\
+	r;					\
+    })
+
 #define CSONPATH_EQUAL_STR(obj, to_cmp)	({			\
     _Bool r = 0;						\
       if (PyUnicode_Check(obj))	{				\
@@ -78,7 +96,7 @@ static int python_set_or_insert_item(PyObject *array,  Py_ssize_t at, PyObject *
 	PyErr_Format(PyExc_ValueError, "Unable to follow path: List expected");
 	return -1;
     }
-    size_t s = PyList_Size(array); 
+    Py_ssize_t s = PyList_Size(array); 
     if (at >= s) {
 	for (;s < at; ++s)
 	    PyList_Insert(array, s, Py_None);
@@ -86,7 +104,8 @@ static int python_set_or_insert_item(PyObject *array,  Py_ssize_t at, PyObject *
 	PyList_Insert(array, at, el);
     } else {
 	Py_INCREF(el);
-	PyList_SetItem(array, at, el);
+	if (PyList_SetItem(array, at, el) < 0)
+	  Py_DECREF(el);
     }
     return 1;
 }
@@ -188,15 +207,7 @@ static PyObject *find_all(PyCsonPathObject *self, PyObject* args)
 
     if (!PyArg_ParseTuple(args, "O", &json))
 	BAD_ARG();
-    PyObject *ret =  csonpath_find_all(self->cp, json);
-    if (ret != Py_None) {
-	PyObject *key_;
-	Py_ssize_t pos_ = 0;
-	PyObject *el;
-	while (PyDict_Next(ret, &pos_, &key_, &el)) {
-	    Py_INCREF(el);
-	}
-    }
+    PyObject *ret = csonpath_find_all(self->cp, json);
     return ret;
 }
 
